import ezkfg as ez


class TestArgs:
    def test_args(self):
        config = ez.load(["--model=resnet18", "--batch-size=32", "--lr=0.01"])

        print(config.dict())

        assert config.model == "resnet18"
        assert config.batch_size == "32"
        assert config.lr == "0.01"
        assert config.get("epoch") == None



import os
from typing import Dict, List, Tuple, Any, Union, Optional, Iterable

from ezkfg import Config


class TestConfig:
    def test_instruction(self):
        config = Config({"a": 1, "b": {"c": 2}, "z.y.x": 233})
        assert config.a == 1
        assert config.b.c == 2
        assert config.z.y.x == 233

        config.a = 3
        assert config.a == 3

        config["d.e.f"] = 3
        assert config["d.e.f"] == 3
        assert config.d.e.f == 3

        config["e.f.g.h"] = 4
        assert config.e.f.g.h == 4
        assert config["e.f.g.h"] == 4

        config.e["h.i.j"] = 5
        assert config.e["h.i.j"] == 5
        assert config.e.h.i.j == 5
        assert config["e.h.i.j"] == 5
        assert config["e.h.i"].get("j", None) == 5

        config["g.h.i.j"] = 7
        assert config.g.h.i.j == 7
        assert config["g.h.i.j"] == 7

        config.load(["--model=resnet18", "--batch-size=32", "--lr=0.01"])
        assert config.model == "resnet18"
        assert config.batch_size == "32"
        assert config.lr == "0.01"

    def test_dict(self):
        config = Config({"a": 1, "b": {"c": 2}, "z.y.x": 233})
        assert isinstance(config.dict(), Dict)
        assert config.dict() == {"a": 1, "b": {"c": 2}, "z": {"y": {"x": 233}}}

    def test_dict_with_config(self):
        config = Config({"c": Config({"d": 1})})
        assert isinstance(config.dict(), Dict)
        assert config.dict() == {"c": {"d": 1}}



import os
import ezkfg as ez


class TestYAML:
    def test_load(self):
        config = ez.load(
            os.path.join(
                os.path.dirname(__file__), "..", "examples", "example_config.yaml"
            )
        )

        print(config.dict())

        assert config.a == 1
        assert config.b.c == 2
        assert config.b.d.e == 3
        assert config.f.g == 4
        assert config.f.h == 5
        assert config.f.i == 6

    def test_dump(self):
        config = ez.load(
            os.path.join(
                os.path.dirname(__file__), "..", "examples", "example_config.yaml"
            )
        )
        config.dump("dump_config.yaml")
        config_ = ez.load("dump_config.yaml")
        for k, v in config.items():
            assert config_[k] == v

        ez.save(config, "dump_config.yaml")
        config_ = ez.load("dump_config.yaml")
        for k, v in config.items():
            assert config_[k] == v



import os
import pickle
import ezkfg as ez


class TestSerialzation:
    def test_serialzation(self):
        config = ez.load(
            os.path.join(
                os.path.dirname(__file__), "..", "examples", "example_config.yaml"
            )
        )
        with open("dump_config.pickle", "wb") as f:
            pickle.dump(config, f)
        with open("dump_config.pickle", "rb") as f:
            config_ = pickle.load(f)
        for k, v in config.items():
            assert config_[k] == v

        assert config_.__frozen__ == config.__frozen__



import os
from pathlib import Path
import ezkfg as ez


class TestJSON:
    file_path = Path(
        os.path.join(os.path.dirname(__file__), "..", "examples", "example_config.json")
    )

    def test_load(self):

        config = ez.load(self.file_path)

        print(config.dict())

        assert config.a == 1
        assert config.b.c == 2
        assert config.b.d.e == 3
        assert config.c.d.e == 3

    def test_dump(self):
        config = ez.load(self.file_path)
        config.dump("dump_config.json")
        config_ = ez.load("dump_config.json")
        for k, v in config.items():
            assert config_[k] == v

        ez.save(config, "dump_config.json")
        config_ = ez.load("dump_config.json")
        for k, v in config.items():
            assert config_[k] == v



import ezkfg as ez


class TestMerge:
    def test_merge(self):
        config1 = ez.load(["--model=resnet18", "--batch-size=32", "--lr=0.01"])
        config2 = ez.load(["--epoch=100", "--lr=0.001"])

        config1.merge(config2, overwrite=False)
        assert config1.model == "resnet18"
        assert config1.batch_size == "32"
        assert config1.lr == "0.01"
        assert config1.epoch == "100"

        config1.merge(config2, overwrite=True)
        assert config1.model == "resnet18"
        assert config1.batch_size == "32"
        assert config1.lr == "0.001"
        assert config1.epoch == "100"



import os
import ezkfg as ez


class TestINI:
    def test_load(self):
        config = ez.load(
            os.path.join(
                os.path.dirname(__file__), "..", "examples", "example_config.ini"
            )
        )

        print(config.dict())

        assert config.a == "1"
        assert config.b == "2"
        assert config.d == "3"
        assert config.e == "4"

    def test_dump(self):
        config = ez.load(
            os.path.join(
                os.path.dirname(__file__), "..", "examples", "example_config.ini"
            )
        )
        config.dump("dump_config.ini")
        config_ = ez.load("dump_config.ini")
        for k, v in config.items():
            assert config_[k] == v

        ez.save(config, "dump_config.ini")
        config_ = ez.load("dump_config.ini")
        for k, v in config.items():
            assert config_[k] == v
